from __future__ import annotations

from collections.abc import Callable
from typing import Any, Optional, Type, TypeVar, Union

from flask import Response

from flask_inputfilter.conditions import BaseCondition
from flask_inputfilter.filters import BaseFilter
from flask_inputfilter.models import ExternalApiConfig, FieldModel
from flask_inputfilter.validators import BaseValidator

T = TypeVar("T")

class InputFilter:
    methods: list[str]
    fields: dict[str, FieldModel]
    conditions: list[BaseCondition]
    global_filters: list[BaseFilter]
    global_validators: list[BaseValidator]
    data: dict[str, Any]
    validated_data: dict[str, Any]
    errors: dict[str, str]
    model_class: Optional[Type[T]]

    def __init__(self, methods: Optional[list[str]] = ...) -> None: ...
    def isValid(self) -> bool: ...
    def is_valid(self) -> bool: ...
    @classmethod
    def validate(
        cls,
    ) -> Callable[
        [Any],
        Callable[
            [tuple[Any, ...], dict[str, Any]],
            Union[Response, tuple[Any, dict[str, Any]]],
        ],
    ]: ...
    def validateData(
        self, data: Optional[dict[str, Any]] = ...
    ) -> Union[dict[str, Any], Type[T]]: ...
    def validate_data(
        self, data: Optional[dict[str, Any]] = ...
    ) -> Union[dict[str, Any], Type[T]]: ...
    def addCondition(self, condition: BaseCondition) -> None: ...
    def add_condition(self, condition: BaseCondition) -> None: ...
    def getConditions(self) -> list[BaseCondition]: ...
    def get_conditions(self) -> list[BaseCondition]: ...
    def setData(self, data: dict[str, Any]) -> None: ...
    def set_data(self, data: dict[str, Any]) -> None: ...
    def getValue(self, name: str) -> Any: ...
    def get_value(self, name: str) -> Any: ...
    def getValues(self) -> dict[str, Any]: ...
    def get_values(self) -> dict[str, Any]: ...
    def getRawValue(self, name: str) -> Any: ...
    def get_raw_value(self, name: str) -> Any: ...
    def getRawValues(self) -> dict[str, Any]: ...
    def get_raw_values(self) -> dict[str, Any]: ...
    def getUnfilteredData(self) -> dict[str, Any]: ...
    def get_unfiltered_data(self) -> dict[str, Any]: ...
    def setUnfilteredData(self, data: dict[str, Any]) -> None: ...
    def set_unfiltered_data(self, data: dict[str, Any]) -> None: ...
    def hasUnknown(self) -> bool: ...
    def has_unknown(self) -> bool: ...
    def getErrorMessage(self, field_name: str) -> Optional[str]: ...
    def get_error_message(self, field_name: str) -> Optional[str]: ...
    def getErrorMessages(self) -> dict[str, str]: ...
    def get_error_messages(self) -> dict[str, str]: ...
    def add(
        self,
        name: str,
        required: bool = ...,
        default: Any = ...,
        fallback: Any = ...,
        filters: Optional[list[BaseFilter]] = ...,
        validators: Optional[list[BaseValidator]] = ...,
        steps: Optional[list[Union[BaseFilter, BaseValidator]]] = ...,
        external_api: Optional[ExternalApiConfig] = ...,
        copy: Optional[str] = ...,
    ) -> None: ...
    def has(self, field_name: str) -> bool: ...
    def getInput(self, field_name: str) -> Optional[FieldModel]: ...
    def get_input(self, field_name: str) -> Optional[FieldModel]: ...
    def getInputs(self) -> dict[str, FieldModel]: ...
    def get_inputs(self) -> dict[str, FieldModel]: ...
    def remove(self, field_name: str) -> Optional[FieldModel]: ...
    def count(self) -> int: ...
    def replace(
        self,
        name: str,
        required: bool = ...,
        default: Any = ...,
        fallback: Any = ...,
        filters: Optional[list[BaseFilter]] = ...,
        validators: Optional[list[BaseValidator]] = ...,
        steps: Optional[list[Union[BaseFilter, BaseValidator]]] = ...,
        external_api: Optional[ExternalApiConfig] = ...,
        copy: Optional[str] = ...,
    ) -> None: ...
    def addGlobalFilter(self, filter: BaseFilter) -> None: ...
    def add_global_filter(self, filter: BaseFilter) -> None: ...
    def getGlobalFilters(self) -> list[BaseFilter]: ...
    def get_global_filters(self) -> list[BaseFilter]: ...
    def clear(self) -> None: ...
    def merge(self, other: InputFilter) -> None: ...
    def setModel(self, model_class: Type[T]) -> None: ...
    def set_model(self, model_class: Type[T]) -> None: ...
    def serialize(self) -> Union[dict[str, Any], T]: ...
    def addGlobalValidator(self, validator: BaseValidator) -> None: ...
    def add_global_validator(self, validator: BaseValidator) -> None: ...
    def getGlobalValidators(self) -> list[BaseValidator]: ...
    def get_global_validators(self) -> list[BaseValidator]: ...
